//
//   main.c
//
//   Copyright 2010 Glibert axel
//
//   This file is part of Deminor.
//
//   Deminor is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Deminor is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Deminor.  If not, see <http://www.gnu.org/licenses/>.
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <SDL.h>
#include "easywiz.h"
#include "deminor.h"

int main (int argc, char *argv[])
{    
    bool screen = true;
    char str[20];
    Uint32 ellapse = 0, now = 0, offset = 0, score[3], nbrBomb = 0, lastKeyPress = 0;
    Coord actualpos = {0, 0};
    GameState state;
    cellule grille[T_GRILLE][T_GRILLE];
    Input in;
    memset(&in, 0, sizeof(in));
    
    wiz_Init();
    wiz_SplashScreen("./gfx/gph.png", 250);
    init_game(T_GRILLE, grille, &nbrBomb, &state, &offset, ellapse, score);
    wiz_LoadImage(IMG_BACKGROUND, "./gfx/back.bmp");
    wiz_LoadImage(IMG_BATTERY, "./gfx/battery.png");
    wiz_LoadTextSurface(IMG_NAME, "Deminor 1.1");
    wiz_LoadTextSurface(IMG_AUTHOR, "axelorque");
    wiz_LoadImage(IMG_TILESET, "./gfx/tileset.png");
    sprintf(str, "Mines: %d", nbrBomb);
    wiz_LoadTextSurface(IMG_MINES, str); 
    wiz_LoadTextSurface(IMG_TOPSCORE, "Top scores");
    sprintf(str, "1: %02d:%02d", score[0]/60, score[0]%60);
    wiz_LoadTextSurface(IMG_SCORE1, str);
    sprintf(str, "2: %02d:%02d", score[1]/60, score[1]%60);
    wiz_LoadTextSurface(IMG_SCORE2, str);
    sprintf(str, "3: %02d:%02d", score[2]/60, score[2]%60);
    wiz_LoadTextSurface(IMG_SCORE3, str); 
    lastKeyPress = SDL_GetTicks();

    while(!in.key[VK_START] == 1){
	
        wiz_DisplayImage(IMG_BACKGROUND, 0, 0); //background        
        wiz_DisplayImagePart(IMG_BATTERY, 295, 5, BATTERY_W, BATTERY_H, (BATTERY_H, wiz_GetBatteryState() - 1) * BATTERY_W, 0);
        wiz_DisplayImage(IMG_NAME, 2, 0); //version
        wiz_DisplayImage(IMG_AUTHOR, 2, 225); //author
        wiz_DisplayImage(IMG_MINES, 240, 20);
        sprintf(str, "Flags left: %d", state.reserveFlags);
        wiz_LoadTextSurface(IMG_FLAGLEFT, str);       
        wiz_DisplayImage(IMG_FLAGLEFT, 240, 40);
        wiz_DisplayImage(IMG_TOPSCORE, 244, 140);
        wiz_DisplayImage(IMG_SCORE1, 250, 153);
        wiz_DisplayImage(IMG_SCORE2, 250, 166);
        wiz_DisplayImage(IMG_SCORE3, 250, 179);      
        affich_grille(T_GRILLE, grille);
        
        if(wiz_UpdateInput(&in))
        {
            lastKeyPress = SDL_GetTicks();
            if(!screen)
            {   /* swich on the screen */
                screen = wiz_SetScreenOnOff(true);
                memset(&in, 0, sizeof(in));  
            }     
        }
		
        if(screen){
		
        if(in.key[VK_SELECT]) // reset game
        {
            init_game(T_GRILLE, grille, &nbrBomb, &state, &offset, ellapse, score);
            sprintf(str, "1: %02d:%02d", score[0]/60, score[0]%60);
            wiz_LoadTextSurface(IMG_SCORE1, str);
            sprintf(str, "2: %02d:%02d", score[1]/60, score[1]%60);
            wiz_LoadTextSurface(IMG_SCORE2, str);
            sprintf(str, "3: %02d:%02d", score[2]/60, score[2]%60);
            wiz_LoadTextSurface(IMG_SCORE3, str);
        }
        
        if(in.key[VK_FA])
        {
            wiz_Screenshot();
            in.key[VK_FA] = 0;
        }
        
        now = SDL_GetTicks();
        if(!state.end) // if game not end
        {
            if(now-(ellapse*1000) >= 1000)
            {
                ellapse++;
            }                    
            update_selector(&actualpos, &in);
            if(in.key[VK_FL] || in.key[VK_FB])
            {
                if(!grille[actualpos.x][actualpos.y].flag && grille[actualpos.x][actualpos.y].hidden)
                    tester_cellule(T_GRILLE, grille, &actualpos, &state, nbrBomb);                
            }
            else if(in.key[VK_FR] || in.key[VK_FX]) 
            {
                if(grille[actualpos.x][actualpos.y].hidden)
                    place_flag(T_GRILLE, grille, &actualpos, &state);
                in.key[VK_FR] = 0;
                in.key[VK_FX] = 0;
            }
            else ;
        }
        else //if game end
        {             
            open_all(T_GRILLE, grille); 
            if(state.win) // if : WIN
            {
                if(!state.gamesaved)
                    save_score(&state, ellapse-offset, score, nbrBomb);
                wiz_LoadTextSurface(IMG_ENDMESSAGE, "SUCCES!");       
            }
            else // if : LOSE
            {
                wiz_LoadTextSurface(IMG_ENDMESSAGE, "GAME OVER"); 
            }
            wiz_DisplayImage(IMG_ENDMESSAGE, 240, 80); 
        }
		
        } //FIN IF(SCREEN) 
		
        if(screen && (now - lastKeyPress > 20000)) // switch off the screen after 20 seconds of inactivity
            screen = wiz_SetScreenOnOff(false);
        sprintf(str, "Time: %02d:%02d", (ellapse-offset)/60, (ellapse-offset)%60);
        wiz_LoadTextSurface(IMG_ENDMESSAGE, str);
        wiz_DisplayImage(IMG_ENDMESSAGE, 240, 220);
        
        wiz_UpdateScreen();
        wiz_framerate(30);
    }
	return 0;
}
